;  This is the test one for the serial led driver
;
;	Flags register layout
;	flags	b'00000000'
;			  |||||||Set for display in F, clear for C
;		      ||||||Set for temp sensor 1, clear for sensor 2
;		      |||||
;		      ||||
;		      |||
;		      ||
;		      |
;			  
;


	LIST	p=16F628
	include	"P16F628.inc"
	__CONFIG _CP_OFF  &_BODEN_OFF &_PWRTE_OFF &_WDT_OFF  &_LVP_OFF &_MCLRE_OFF &_INTRC_OSC_NOCLKOUT


; The defines
#DEFINE clock 	PORTA, 0
#DEFINE datout 	PORTA, 1
#DEFINE latch 	PORTA, 2

;#DEFINE	but1	PORTB, 7
#DEFINE thing	PORTB, 0

	CONSTANT DATA_PIN_0 = 3
	CONSTANT DATA_PIN_1 = 4

	cblock	0x20
		D1		;Segment 4	
		D2		;Segment 3
		D3		;Segment 2
		D4		;Segment 1
		t0		;Temps for delay routines
		t1
		t2
		t3
		temp	;Holds value to be put on the SPI and 1-wire bytes
		flags	;flags to control control		
		counter

	  	hundreds		;Holds hundreds after conversion
	  	tens_and_ones	;Holds tens and ones after conversion
		bin				;Holds value for conversion
		temp_lsb
		temp_msb
		temp_C1			;Holds temp in C
		temp_F1			;Holds temp in F
		temp_C2			;Holds temp in C
		temp_F2			;Holds temp in F

		;1-wire varibles
		index	;Loop contorl register
		o_byte	;Address of output byte
		i_byte	;Address of input byte

	endc

;Define ram locations
	org		0x00
	goto	init

;--------------------------------------------------------------------
;**************************** SUBROUTINES ***************************


;******************************* B2L ********************************           
; This table is used to convert BCD to LED values for the display
B2L:   
		addwf   PCL,F
		;		b'abcdefgX'
        retlw   b'11111100'     ; 0
        retlw   b'01100000'     ; 1
        retlw   b'11011010'     ; 2
        retlw   b'11110010'     ; 3
        retlw   b'01100110'     ; 4
        retlw   b'10110110'     ; 5
        retlw   b'10111110'     ; 6
        retlw   b'11100000'     ; 7
        retlw   b'11111110'     ; 8
        retlw   b'11110110'     ; 9
        retlw   b'10011100'     ; C = 10
        retlw   b'10001110'     ; F = 11
        retlw   b'11000110'     ; "Degree sign" = 12
        retlw   b'00000010'     ; - = 13
;**************************** End B2L *******************************

;**************************** F_conv ********************************
;Jump table for ferenhiht conversion
F_conv:
	addwf	PCL, F
	;		b'nnnnnnnd'
	retlw	b'10001000'		;68f	20c
	retlw	b'10001010'		;69f	20.5c
	retlw	b'10001100'		;70f	21c
	retlw	b'10001101'		;70.5f	21.5c
	retlw	b'10001111'		;71.5f	22c
	retlw	b'10010001'		;72.5f	22.5c
	retlw	b'10010011'		;73.5f	23c							
	retlw	b'10010100'		;74f	23.5c							
	retlw	b'10010110'		;75f	24c							
	retlw	b'10011000'		;76f	24.5c							
	retlw	b'10011010'		;77f	25c
	retlw	b'10011100'		;78f	25.5c
	retlw	b'10011110'		;79f	26c
	retlw	b'10100000'		;80f	26.5c
	retlw	b'10100001'		;80.5f	27c
	retlw	b'10100011'		;81.5f	27.5c
	retlw	b'10100101'		;82.5f	28c
	retlw	b'10100111'		;83.5f	28.5c
	retlw	b'10101000'		;84f	29c
	retlw	b'10101010'		;85f	29.5c
	retlw	b'10101100'		;86f	30c
	retlw	b'10101110'		;87f	30.5c
	retlw	b'10110000'		;88f	31c
	retlw	b'10110010'		;89f	31.5c
	retlw	b'10110100'		;90f	32c
;************************ End F_conv ********************************


;***************************** latch_it *****************************
; Used to latch the output line ofthe serial led driver
latch_it
	bcf		latch
	nop
	bsf		latch
	nop
	bcf		latch
	return
;*************************** End latch_it ***************************

;******************************* Disp *******************************
; Subroutine will push the data in the digits to the serial latch 
; that drive the segmented displays
; Data to be pushed is in D1,D2,D3 and D4
Disp
	movf	D1, w
	call	ser_out
	movf	D2, w
	call	ser_out
	movf	D3, w
	call	ser_out	
	movf	D4, w
	call	ser_out	

	call	latch_it
	return
;***************************** End Disp *****************************


;***************************** ser_out ******************************
; This routine outputs single bytes on the SPI bus
; w holds value to be pushed out
ser_out
	movwf	temp
	movlw	d'8'			;Load counter for one byte output
	movwf	counter
	movlw	d'0'

nextbit
	movf	temp, w			;Load value to shift out
	btfsc	temp, 0 		;Is current output a one?
	goto	output_1		;Yes output 1
	goto	output_0		;No output 0

output_0
	bcf		datout
	nop
	nop
	bsf		clock
	nop
	bcf		clock
	goto	rest

output_1
	bsf		datout			;clock output line
	nop
	nop
	bsf		clock
	nop
	bcf		clock
	bcf		datout

rest	
	rrf		temp, w		;Rotate Right
	movwf	temp
	decfsz	counter,f		;Are we done yet
	goto	nextbit

	return
;************************** End ser_out *****************************


;************************* binary_to_bcd ****************************
; Do the binary to BCD conversion
; Valued to be converted are in bin and are returned in hundreds
; and tens_and_ones
binary_to_bcd:
	clrf	hundreds		
	swapf	bin, W
	addwf	bin, W
	andlw	B'00001111'
	skpndc
	  addlw 0x16
	skpndc
	  addlw 0x06
	addlw 	0x06
	skpdc
	addlw 	-0x06
	btfsc	bin, 4
	  addlw 0x16 - 1 + 0x6
	skpdc
	  addlw -0x06
	btfsc	bin, 5
	  addlw 0x30
	btfsc	bin, 6
	  addlw 0x60
	btfsc	bin, 7
	  addlw	0x20
	addlw	0x60
	rlf		hundreds, f
	btfss	hundreds, 0
	  addlw	-0x60
	movwf	tens_and_ones
	btfsc	bin, 7
	  incf	hundreds, f
	return
;********************** End binary_to_bcd ***************************


;*******************************************************************        
;***** Delay Routines      
; 500 us delay @ 4MHz
D500us  
		movlw	0xA6		;499 cycles
		movwf	t0
D500u
		decfsz	t0, f
		goto	D500u
		nop					;1 cycle
        return

; 60 us delay @ 4MHz
D60us   
		movlw	0x13		;58 cycles
		movwf	t0
D60
		decfsz	t0, f
		goto	D60
		goto	$+1			;2 cycles
        return

;*** 1 ms delay @ 4MHz        
D1ms    
		movlw	0xC7		;998 cycles
		movwf	t0
		movlw	0x01
		movwf	t1
D1m
		decfsz	t0, f
		goto	$+2
		decfsz	t1, f
		goto	D1m
		goto	$+1			;2 cycles
        return 

;"Normal" Delay  500ms @ 4MHz
D500ms  
		movlw	0x03		;499994 cycles
		movwf	t0
		movlw	0x18
		movwf	t1
		movlw	0x02
		movwf	t2
D500
		decfsz	t0, f
		goto	$+2
		decfsz	t1, f
		goto	$+2
		decfsz	t2, f
		goto	D500
		goto	$+1			;6 cycles
		goto	$+1
		goto	$+1
		return
;************************** End Delay Routines **********************

;********************************************************************
;***  DS COMMUNICATION
;The following are standerd 1-wire routines.
;THIS IS FOR THE SENSOR ON DATA_PIN_0
;Initilize the DS1820
initds1820_0:
	call	pin_hi_0
	call	pin_lo_0
	call	D500us
	call	pin_hi_0
	call	D500us
	return

;Check if the DS1820 is done converting
test_0:
	call	in_byte_0
	movlw	h'FF'
	subwf	i_byte, W
	return

;Recieve a byte
in_byte_0:
	movlw	d'8'
	movwf	index
	clrf	i_byte

in_byte_1_0:
	call	pin_lo_0
	nop					;Wait 1us
	call	pin_hi_0
	nop
	nop
	nop
	nop
	nop
	nop
	nop					;Wait 7us
	movf	PORTA, W
	movwf	temp
	btfss	temp, DATA_PIN_0
	bcf		STATUS, C
	btfsc	temp, DATA_PIN_0
	bsf		STATUS, C
	rrf		i_byte, f
	call	D60us  
	decfsz	index, f
	goto	in_byte_1_0
	movfw	i_byte
	return

;Output a byte on the one wire
out_byte_0:
	movlw	d'8'
	movwf	index

out_byte_1_0:	
	rrf		o_byte, f
	btfss	STATUS, C
	goto	out_0_0
	goto	out_1_0

out_byte_2_0:
	decfsz	index, f
	goto	out_byte_1_0
	return

out_0_0:
	call	pin_lo_0
	call	D60us  
	call	pin_hi_0
	goto	out_byte_2_0

out_1_0:
	call	pin_lo_0
	call	pin_hi_0
	call	D60us  
	goto	out_byte_2_0

;Set the 1-wire bus high
pin_hi_0:
	bsf		STATUS, RP0
	bsf		TRISA, DATA_PIN_0
	bcf		STATUS, RP0

	return

;Set the 1-wire bus low
pin_lo_0:
	bcf		PORTA, DATA_PIN_0
	bsf		STATUS, RP0
	bcf		TRISA, DATA_PIN_0
	bcf		STATUS, RP0

	return
;********************************************************************
;THIS IS FOR THE SENSOR ON DATA_PIN_1
;Initilize the DS1820
initds1820_1:
	call	pin_hi_1
	call	pin_lo_1
	call	D500us
	call	pin_hi_1
	call	D500us
	return

;Check if the DS1820 is done converting
test_1:
	call	in_byte_1
	movlw	h'FF'
	subwf	i_byte, W
	return

;Recieve a byte
in_byte_1:
	movlw	d'8'
	movwf	index
	clrf	i_byte

in_byte_1_1:
	call	pin_lo_1
	nop					;Wait 1us
	call	pin_hi_1
	nop
	nop
	nop
	nop
	nop
	nop
	nop					;Wait 7us
	movf	PORTA, W
	movwf	temp
	btfss	temp, DATA_PIN_1
	bcf		STATUS, C
	btfsc	temp, DATA_PIN_1
	bsf		STATUS, C
	rrf		i_byte, f
	call	D60us  
	decfsz	index, f
	goto	in_byte_1_1
	movfw	i_byte
	return

;Output a byte on the one wire
out_byte_1:
	movlw	d'8'
	movwf	index

out_byte_1_1:	
	rrf		o_byte, f
	btfss	STATUS, C
	goto	out_0_1
	goto	out_1_1

out_byte_2_1:
	decfsz	index, f
	goto	out_byte_1_1
	return

out_0_1:
	call	pin_lo_1
	call	D60us  
	call	pin_hi_1
	goto	out_byte_2_1

out_1_1:
	call	pin_lo_0
	call	pin_hi_0
	call	D60us  
	goto	out_byte_2_1

;Set the 1-wire bus high
pin_hi_1:
	bsf		STATUS, RP0
	bsf		TRISA, DATA_PIN_1
	bcf		STATUS, RP0

	return

;Set the 1-wire bus low
pin_lo_1:
	bcf		PORTA, DATA_PIN_1
	bsf		STATUS, RP0
	bcf		TRISA, DATA_PIN_1
	bcf		STATUS, RP0

	return
;********************************************************************


;**************************** Disp_temp *****************************
; Subroutine will display temp on the display 
Disp_temp:
	clrf	temp_lsb
	clrf	temp_msb
	clrf	temp
	clrf	i_byte
	clrf	o_byte

	movwf	flags
	btfss	flags, 1		;test to see which temp to read
	goto	Geta
	goto	Getb

Geta
	call 	Get_tempa		;Read the tempurateure sensor
	goto	Next
Getb
	call	Get_tempb

Next
	btfss	flags, 0 		
	goto	Cel

Fer
	movf	temp_lsb, w		;Load temp_lsb
	sublw	d'40'			;Subtract 40 to create jump (40-lsb)
	subwf	0, W			;Invert value to be -(40-lsb)
	call	F_conv			;Call conversion table
	movwf	temp_lsb		;Store tempurature in ferheinght

Cel
	movf	temp_lsb, w
	rrf		temp_lsb, w
	movwf	bin
	call	binary_to_bcd

	movf	tens_and_ones, W	
	swapf	tens_and_ones, W
	andlw	0x0F
	call	B2L
	movwf	D4
	
	movf	tens_and_ones, W
	andlw	B'00001111'
	call	B2L
	movwf	D3

	btfss	flags, 0 
	goto	Celcius

Ferehnit
	movlw	.11	;F
	call	B2L
	movwf	D1
	goto	Degree

Celcius
	movlw	.10	;C
	call	B2L
	movwf	D1
	goto	Degree

Degree
	movlw	.12	;degree symbol
	call	B2L
	movwf	D2
	
	call	Disp
	return
;************************** End Disp_temp ***************************


;****************************** Get_tempa ***************************
;  This will read the temp from the DS1820 on DATA_PIN_0
Get_tempa:
	clrf	temp_lsb
	clrf	temp_msb
	clrf	temp
	clrf	i_byte
	clrf	o_byte

	call	initds1820_0	;Initilize the ds1820 

	movlw	h'CC'			;Send skip ROM command
	movwf	o_byte
	call	out_byte_0

	movlw	h'44'			;Send start conversion command
	movwf	o_byte
	call	out_byte_0

Wait_0:
	call	test_0
	btfss	STATUS, Z
	goto	Wait_0

	call	initds1820_0	;Reinitilize the DS1820

	movlw	h'CC'			;Send skip ROM command
	movwf	o_byte
	call	out_byte_0

	movlw	h'BE'			;Send command
	movwf	o_byte
	call	out_byte_0

	call	in_byte_0		;Read in tempurature LSB
	movwf	temp_lsb

	call	in_byte_0		;Read in tempurature MSB
	movwf	temp_msb

	return
;************************** End Get_tempa ***************************


;****************************** Get_tempa ***************************
;  This will read the temp from the DS1820 on DATA_PIN_0
Get_tempb:
	clrf	temp_lsb
	clrf	temp_msb
	clrf	temp
	clrf	i_byte
	clrf	o_byte

	call	initds1820_1	;Initilize the ds1820 

	movlw	h'CC'			;Send skip ROM command
	movwf	o_byte
	call	out_byte_1

	movlw	h'44'			;Send start conversion command
	movwf	o_byte
	call	out_byte_1

Wait_1:
	call	test_1
	btfss	STATUS, Z
	goto	Wait_1

	call	initds1820_1	;Reinitilize the DS1820

	movlw	h'CC'			;Send skip ROM command
	movwf	o_byte
	call	out_byte_1

	movlw	h'BE'			;Send command
	movwf	o_byte
	call	out_byte_1

	call	in_byte_1		;Read in tempurature LSB
	movwf	temp_lsb

	call	in_byte_1		;Read in tempurature MSB
	movwf	temp_msb

	return
;************************** End Get_tempb ***************************


;******************************* Hello ******************************
; SCROLL "HELLO"

HELLO:  
		MOVLW   b'00000000'     ; "    "
        MOVWF   D4
        MOVWF   D3
        MOVWF   D2
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVLW   b'01101110'     ; "   H"
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D1,W            ; "  HE"
        MOVWF   D2
        MOVLW   b'10011110'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D2,W            ; " HEL"
        MOVWF   D3
        MOVF    D1,W
        MOVWF   D2
        MOVLW   b'00011100'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D3,W            ; "HELL"
        MOVWF   D4
        MOVF    D2,W
        MOVWF   D3
        MOVF    D1,W
        MOVWF   D2
        MOVLW   b'00011100'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D3,W            ; "ELLO"
        MOVWF   D4
        MOVF    D2,W
        MOVWF   D3
        MOVF    D1,W
        MOVWF   D2
        MOVLW   b'11111100'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D3,W            ; "LLO "
        MOVWF   D4
        MOVF    D2,W
        MOVWF   D3
        MOVF    D1,W
        MOVWF   D2
        MOVLW   b'00000000'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D3,W            ; "LO  "
        MOVWF   D4
        MOVF    D2,W
        MOVWF   D3
        MOVF    D1,W
        MOVWF   D2
        MOVLW   b'00000000'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D3,W            ; "O   "
        MOVWF   D4
        MOVF    D2,W
        MOVWF   D3
        MOVF    D1,W
        MOVWF   D2
        MOVLW   b'00000000'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms

        MOVF    D3,W            ; "    "
        MOVWF   D4
        MOVF    D2,W
        MOVWF   D3
        MOVF    D1,W
        MOVWF   D2
        MOVLW   b'00000000'
        MOVWF   D1
        CALL    Disp
        CALL    D500ms
        RETURN
;************************** End Hello *******************************

;************************* END SUBROUTINES **************************
;********************************************************************


;Setup stuff
init
	movlw	0x07		;Turn off the analog comparators
	movwf	CMCON

	bsf		STATUS, RP0	;Goto the right page
	
	movlw	b'00011000'	;Make port A4,3 input all others output
	movwf	TRISA 		
	clrf	TRISB		;Make Port B all outputs
	bcf		STATUS, RP0	;Go back to the oter page
	bcf		clock	
	bcf		datout
	bcf		latch

;********************* Begin the main Program HERE*******************
;This is the main program
main

	call	D500ms		;Wait 2 secs for things to settle
	call	D500ms

	;call	HELLO		;Scroll hello on the display
splat
	bcf		thing
	movlw	b'00000000'	;Call with 0 in w for celcius
	call	Disp_temp	;1 for ferenhight

	call	D500ms		;Wait 2 secs
	call	D500ms
	call	D500ms
	call	D500ms	

	movlw	b'00000001'	;Call with 0 in w for celcius
	call	Disp_temp	;1 for ferenhight

	call	D500ms		;Wait 2 secs
	call	D500ms
	call	D500ms
	call	D500ms	

	bsf		thing
	movlw	b'00000010'	;Call with 0 in w for celcius
	call	Disp_temp	;1 for ferenhight

	call	D500ms		;Wait 2 secs
	call	D500ms
	call	D500ms
	call	D500ms	

	movlw	b'00000011'	;Call with 0 in w for celcius
	call	Disp_temp	;1 for ferenhight

	call	D500ms		;Wait 2 secs
	call	D500ms
	call	D500ms
	call	D500ms	

	goto	splat
	goto	main		;Goto main and start next conversion


	end


